@@ -3,9 +3,7 @@ use warnings;
 
 package App::dategrep;
 use App::dategrep::Date qw(intervall_to_epoch date_to_epoch minutes_ago);
-use App::dategrep::Iterator::File;
-use App::dategrep::Iterator::Stdin;
-use App::dategrep::Iterator::Uncompress;
+use App::dategrep::Iterators;
 use Config::Tiny;
 use Pod::Usage;
 use Getopt::Long;
@@ -14,7 +12,7 @@ use File::Basename qw(basename);
 use base 'Exporter';
 our @EXPORT_OK = qw(run);
 
-our $VERSION = '0.54';
+our $VERSION = '0.56';
 
 our $app;
 
@@ -40,7 +38,7 @@ sub run {
         \%options,        'start|from=s', 'end|to=s',     'format=s',
         'last-minutes=i', 'multiline!',   'blocksize=i',  'help|?',
         'sort-files',     'man',          'configfile=s', 'interleave',
-        'byte-offsets',   'debug=s', 'version!',
+        'byte-offsets',   'debug=s', 'version!', 'skip-unparsable!',
     );
     if ( !$rc ) {
         pod2usage( -exitstatus => "NOEXIT", -verbose => 0 );
@@ -81,6 +79,10 @@ sub run {
         $options{'format'} = $named_formats{ $options{'format'} };
     }
 
+    if ( $options{'skip-unparsable'} ) {
+	    $options{'multiline'} = 0;
+    }
+
     my ( $start, $end ) = ( 0, time() );
 
     if ( defined $options{'start'} ) {
@@ -130,20 +132,23 @@ sub run {
             }
         }
 
-        my @iterators =
-          map { get_iterator( $_, $start, $end, %options ) } @ARGV;
-
+        my $iterators = App::dategrep::Iterators->new(
+            %options,
+            filenames => \@ARGV,
+            start     => $start,
+            end       => $end,
+        );
 
         if ( $options{'interleave'} ) {
-            interleave_iterators( $options{'format'}, @iterators );
+            $iterators->interleave();
             return 0;
         }
 
         if ( $options{'sort-files'} ) {
-            @iterators = sort_iterators( $options{'format'}, @iterators );
+            $iterators->sort();
         }
 
-        for my $iter (@iterators) {
+        for my $iter ($iterators->as_array) {
             if ($iter) {
                 while ( my $entry = $iter->get_entry ) {
                     print $entry;
@@ -155,75 +160,6 @@ sub run {
     return 0;
 }
 
-sub guess_format {
-    my ($formats, @iterators) = @_;
-    for my $iterator (@iterators) {
-        my $line = $iterator->peek;
-        for my $format ( @$formats ) {
-            my $epoch = date_to_epoch( $line, $format );
-            if ( defined $epoch ) {
-                return $format;
-            }
-        }
-    }
-    return;
-}
-
-sub interleave_iterators {
-    my ( $format, @iterators ) = @_;
-
-    while ( @iterators = sort_iterators( $format, @iterators ) ) {
-        print $iterators[0]->get_entry;
-    }
-    return;
-}
-
-sub get_iterator {
-    my ( $filename, $start, $end, %options ) = @_;
-    my ( $multiline, $format ) = @options{qw(multiline format)};
-    my @args = (
-        start     => $start,
-        end       => $end,
-        multiline => $multiline,
-        format    => $format
-    );
-    my $iter;
-    if ( $filename eq '-' ) {
-        $iter = App::dategrep::Iterator::Stdin->new(@args);
-    }
-    elsif ( $filename =~ /\.(bz|bz2|gz|z)$/ ) {
-        $iter =
-          App::dategrep::Iterator::Uncompress->new( @args,
-            filename => $filename );
-    }
-    else {
-        $iter =
-          App::dategrep::Iterator::File->new( @args, filename => $filename );
-    }
-    return $iter;
-}
-
-sub sort_iterators {
-    my ( $format, @iterators ) = @_;
-
-    my @timestamps;
-    for my $iterator (@iterators) {
-        my $line = $iterator->peek;
-        
-        ## remove all iterators with eof
-        next if not defined $line;
-
-        ## TODO What should we do under --multiline?
-        my ( $epoch, $error ) = date_to_epoch( $line, $format );
-        if ( !$epoch ) {
-            ## TODO Which iterator produced the error?
-            die "No date found in first line: $error\n";
-        }
-        push @timestamps, [ $epoch, $iterator ];
-    }
-    return map { $_->[1] } sort { $a->[0] <=> $b->[0] } @timestamps;
-}
-
 sub loadconfig {
     my $configfile = shift;
     if ( not $configfile and $ENV{HOME} ) {
