@@ -4,7 +4,6 @@ use warnings;
 use Fcntl ":seek";
 use Moo;
 use FileHandle;
-use App::dategrep::Date qw(date_to_epoch);
 extends 'App::dategrep::Iterator';
 
 has 'filename' => ( is => 'ro', required => 1 );
@@ -22,21 +21,20 @@ sub _build_fh {
     my ( $fh, $tell_beg, $tell_end ) = $self->byte_offsets();
     $self->tell_beg($tell_beg);
     $self->tell_end($tell_end);
-    if ( defined($tell_beg) ) {
-        $fh->seek( $tell_beg, SEEK_SET );
-    }
+    $fh->seek( $tell_beg, SEEK_SET );
     return $fh;
 }
 
-sub get_entry {
+sub get_entry_unbuffered {
     my $self = shift;
     my $line = $self->getline();
     ## TODO can $tell_end be undefined?
     return
       if defined( $self->tell_end ) && ( $self->fh->tell > $self->tell_end );
-    if ( $self->multiline && ! $self->fh->eof && !$self->next_line_has_date ) {
-
-        $line .= $self->getline();
+    if ( $self->multiline ) {
+	while ( !$self->fh->eof && !$self->next_line_has_date ) {
+		$line .= $self->getline();
+	}
     }
     return $line;
 }
@@ -47,7 +45,7 @@ sub byte_offsets {
     open( my $fh, '<', $filename ) or die "Can't open $filename: $!\n";
     my $test_line = $fh->getline;
     if ( defined($test_line) ) {
-        my ( $epoch, $error ) = date_to_epoch( $test_line, $self->format );
+        my ( $epoch, $error ) = $self->to_epoch( $test_line );
         if ($error) {
             die "No date found in first line: $error\n";
         }
@@ -87,9 +85,9 @@ sub search {
         $fh->seek($mid * $blksize, 0 ) or return;
         $fh->getline if $mid;    # probably a partial line
       LINE: while ( my $line = $fh->getline() ) {
-            my ($epoch) = date_to_epoch( $line, $self->format );
+            my ($epoch) = $self->to_epoch( $line );
             if ( !$epoch ) {
-                next LINE if $multiline;
+                next LINE if $multiline || $self->skip_unparsable;
 
                 chomp($line);
                 die "Unparsable line: $line\n";
@@ -112,9 +110,9 @@ sub search {
     for ( ; ; ) {
         $min = $fh->tell;
         defined( my $line = $fh->getline ) or last;
-        my ($epoch) = date_to_epoch( $line, $self->format );
+        my ($epoch) = $self->to_epoch( $line );
         if ( !$epoch ) {
-            next if $multiline;
+            next if $multiline || $self->skip_unparsable;
             chomp($line);
             die "Unparsable line: $line\n";
         }
